/**
 * @file
 * @preserve
 * DOM content synchronization with Mustache templates.
 *
 * Using Mustache.js by
 *   Copyright (c) 2009 Chris Wanstrath (Ruby)
 *   Copyright (c) 2010-2014 Jan Lehnardt (JavaScript)
 *   Copyright (c) 2010-2015 The mustache.js community
 */

window.mustacheSync = window.mustacheSync || {};
window.mustacheSync.items = window.mustacheSync.items || [];

(function (Drupal, sync, Mustache, window) {
  sync.providers = sync.providers || {};
  sync.registry = sync.registry || [];
  sync.listeners = sync.listeners || {
    __callback: function () {
      var i;
      var subscriber;
      var period;
      var length = this.subscribers.length;
      for (i = 0; i < length; i++) {
        subscriber = this.subscribers[i];
        period = subscriber.item.period;
        if (!(subscriber.limit === 0)) {
          subscriber.limit--;
          if (subscriber.item.triggered === true) {
            period = 0;
          }
          subscriber.item.triggered = true;
          subscriber.item.sync(subscriber.item.delay, period, true);
        }
      }
    }
  };

  sync.now = sync.now || function () {
    var item;

    sync.___update = sync.___update || function (period, force_fetch) {
      var provider = this.provider;
      var fetch = sync.___fetch;
      var render = sync.___render;
      if (this.ready !== true) {
        return;
      }
      if (typeof period === 'undefined') {
        period = this.period;
      }
      if (provider.faulty === true) {
        if (period > 0) {
          // Retry when period is set.
          this.sync(this.delay + 1000, period);
        }
        return;
      }
      if (provider.fetching === true) {
        // Retry.
        this.sync(this.delay + 10, period);
        return;
      }
      if ((provider.fetched === null) || ((period > 0) && (period - 1 < Date.now() - provider.fetched)) || (force_fetch === true)) {
        fetch.call(this, period);
      }
      else {
        render.call(this);
        if (period > 0) {
          // Repeat.
          this.sync(period, period);
        }
      }
    };
    sync.___fetch = sync.___fetch || function (period) {
      var provider = this.provider;
      var render = sync.___render;
      var request = new XMLHttpRequest();

      provider.fetching = true;
      if (typeof period === 'undefined') {
        period = this.period;
      }

      request.open('GET', provider.url, true);
      request.onload = function () {
        var data;
        if (request.status >= 200 && request.status < 400) {
          try {
            data = JSON.parse(request.responseText);
          }
          catch (e) {
            data = false;
          }
          provider.latest = data;
          provider.fetched = Date.now();
          provider.fetching = false;
          provider.faulty = false;

          render.call(this);
          if (period > 0) {
            // Repeat.
            this.sync(period, period);
          }
        }
        else {
          request.onerror();
        }
      }.bind(this);
      request.onerror = function () {
        provider.fetching = false;
        provider.faulty = true;
        if (period > 0) {
          // Retry when period is set.
          window.setTimeout(sync.___fetch.bind(this, period), period + 5000);
        }
      }.bind(this);

      request.send();
    };
    sync.___render = sync.___render || function () {
      var el = this.element;
      var data = this.provider.latest;
      var select = this.select;
      var subset = sync.___subset;

      if (select !== null && typeof select === 'object' && data !== null && typeof data === 'object') {
        data = subset(data, select);
      }
      if (typeof data === 'object') {
        el.innerHTML = Mustache.render(this.template, data);
        if (el.classList) {
          el.classList.add('synced');
          el.classList.remove('not-synced');
        }
        else {
          el.className += ' synced';
          el.className = el.className.replace(new RegExp('(^|\\b)not-synced(\\b|$)', 'gi'), ' ');
        }
      }
    };
    sync.__init = sync.__init || function () {
      var providers = sync.providers;
      if (this.ready === true) {
        // Already initialized, aborting.
        return;
      }

      // Initialize the corresponding provider.
      if (!providers.hasOwnProperty(this.src)) {
        providers[this.src] = {
          url: this.src,
          latest: {},
          fetched: null,
          fetching: false,
          faulty: false
        };
      }
      this.provider = providers[this.src];

      this.sync = this.sync || function (delay, period, force_fetch) {
        if (typeof delay === 'undefined') {
          delay = this.delay;
        }
        if (typeof period === 'undefined') {
          period = this.period;
        }
        if (force_fetch !== true) {
          force_fetch = false;
        }
        window.setTimeout(sync.___update.bind(this, period, force_fetch), delay);
      }.bind(this);

      this.listen = this.listen || function (triggers) {
        var trigger;
        var selector;
        var event;
        var limit;
        var group;
        var subscribers;
        var i;

        // Add this item as a subscriber for each specified triggering element.
        for (i = 0; i < triggers.length; i++) {
          trigger = triggers[i];
          selector = trigger[0];
          event = trigger[1];
          limit = trigger[2];
          if (!sync.listeners.hasOwnProperty(selector)) {
            sync.listeners[selector] = {
              elements: [],
              events: {}
            };
          }
          group = sync.listeners[selector];
          if (!group.events.hasOwnProperty(event)) {
            group.events[event] = {
              callback: sync.listeners.__callback,
              subscribers: []
            };
          }
          subscribers = group.events[event].subscribers;
          subscribers.push({item: this, limit: limit});
        }
      }.bind(this);

      this.delay = this.delay || 0;
      this.period = this.period || 0;
      this.trigger = this.trigger || null;

      this.started = false;
      this.triggered = false;
      this.ready = true;
    };
    sync.__start = sync.__start || function () {
      if (this.ready !== true || this.started === true) {
        // Already started synchronization or not initialized, aborting.
        return;
      }
      this.started = true;

      if (this.trigger === null) {
        // Synchronize immediately, or at least after a specified delay.
        this.sync(this.delay, this.period);
      }
      else {
        // Listen and synchronize by the specified triggers.
        this.listen(this.trigger);
      }
    };

    while (sync.items.length > 0) {
      item = sync.items.shift();
      sync.__init.call(item);
      sync.registry.push(item);
      sync.__start.call(item);
    }
  };
  sync.refresh = sync.refresh || function (dom) {
    var selector;
    var group;
    var new_elements;
    var registered;
    var event;
    var i;
    var el;
    var length;

    sync.now();

    if (typeof dom === 'undefined') {
      dom = window.document;
    }

    // Collect all triggering elements,
    // and create the corresponding event listeners.
    for (selector in sync.listeners) {
      if (!sync.listeners.hasOwnProperty(selector)) {
        continue;
      }
      group = sync.listeners[selector];

      if (dom !== null) {
        new_elements = dom.querySelectorAll(selector);
        length = new_elements.length;
        for (i = 0; i < length; i++) {
          el = new_elements[i];
          if (group.elements.indexOf(el) < 0) {
            group.elements.push(el);
          }
          for (event in group.events) {
            if (!group.events.hasOwnProperty(event)) {
              continue;
            }
            registered = group.events[event];

            length = group.elements.length;
            for (i = 0; i < length; i++) {
              group.elements[i].addEventListener(event, registered.callback.bind(registered), false);
            }
          }
        }
      }
    }
  };
  sync.___subset = sync.___subset || function (data, select) {
    var subset = data;
    var slength = select.length;
    var skey;
    var i;
    for (i = 0; i < slength; i++) {
      skey = select[i];
      if (subset.hasOwnProperty(skey)) {
        subset = subset[skey];
      }
      else {
        return false;
      }
    }
    return subset;
  };

  sync.now();

  Drupal.behaviors.mustacheSync = {
    attach: function attach(dom) {
      sync.refresh(dom);
    }
  };
}(Drupal, window.mustacheSync, window.Mustache, window));
